/*	##############################################################################################
	Advanced RTI System, ARTÌS			http://pads.cs.unibo.it

	Description:	
		Example of using ARTÌS with Java. It simulates a set of nodes in a ring. The nodes send
		and receive interactions from other nodes in the ring.

	Output:
		the output can be found in the following files:
		<LP_ID>.out	model output
		<LP_ID>.err	middleware output

	Changelog:
		13/10/09
			A little of commenting

	Authors:
		First version by Gabriele D'Angelo <gda@cs.unibo.it>

	############################################################################################## */

class JTS_ring {

	//	The simulation ends at this simulated time
	final static int	END_CLOCK = 20;


	//	Main procedure
	public static void main(String args[]) {
  
		//	Simulated time
		double	clock = 0;
		//	LP identifier
		int   	LP_id;
		//	Number of LPs in the ring
		int	nLP=0;

		// 	Create the class instance
		ARTISjava javaOBJ=new ARTISjava();

 		//	Get as a parameter the number of nodes in the ring
		nLP	= Integer.parseInt(args[1]);
		//	Initialize the synchronization mechanism and get the local LP identifier
		LP_id	= javaOBJ.TS_Init(args[0], "localhost", 5000);
		
		//	If the LP is the first one created then bootstrap the interation cycle
		if (LP_id == 0)
			javaOBJ.TS_Send(1, 1.0, "PING", 4);	// send a message to LP 1
		
		//	Main simulation cycle
		while (clock < END_CLOCK) {

			//	Receive a new message (if available)
			String data = new String(javaOBJ.TS_Receive(10));
					
			//	Check the return value of TS_Receive()
			//		if (ready > 0) then 	it is the message size
			//		else			we have no more messages to receive
			if (javaOBJ.ready > 0) {

				//	Output the received message
				System.out.println("Time ["+javaOBJ.Ts+"] Message from "+ javaOBJ.from + " ("+data+")");
		
				//	Define a destination of a new message to be created
				int dest = (LP_id + 1) % nLP;

				//	Send the new message
				javaOBJ.TS_Send(dest, clock + 1.0, "PING", 4);
			}  
			else  {
				//	If no more messages are in the queue
				//
				//	Proceed with the next timestep
				clock = javaOBJ.TS_TimeAdvance();
			}
			
		}

		//	The simulation is finished
		System.out.println("["+clock+"] --- LP "+LP_id+" Finished!");

		//	Finalize the synchronization mechanism
		javaOBJ.TS_Finalize();		
	}
}
