/*	##############################################################################################
	Advanced RTI System, ARTÌS			http://pads.cs.unibo.it

	Description:	
		Constants, data structures and prototypes of gaia.c used by the simulation model layer.

	Authors:
		First version by Michele Bracuto <bracuto@cs.unibo.it>
		Revised by Gabriele D'Angelo <gda@cs.unibo.it>

	############################################################################################### */

#ifndef __GAIA_H
#define __GAIA_H

/*---------------------------------------------------------------------------*/

#include <ts.h>

/*---------------------------------------------------------------------------*/
/*	T Y P E    D E F I N I T I O N S				     */	
/*---------------------------------------------------------------------------*/

typedef	unsigned int	u_int;

/*---------------------------------------------------------------------------*/
/*	C O N S T A N T S						     */	
/*---------------------------------------------------------------------------*/

#define	UNSET			'U'	// Model Level Event
#define	EOS			'E'	// End Of Step (EOS)
#define	STATE_CHANGE		'A'	// TODO
#define	REQUIRE_MIGR		'Y'	// TODO
#define	NOTIF_MIGR		'N'	// Notification of Migration
#define	NOTIF_MIGR_EXT		'n'	// Notification of External Migration
#define	EXEC_MIGR		'M'	// Execute Migration
#define	REGISTER		'R'	// Register a new Simulated Entity

#define	MIGR_OFF		0	// Migration OFF
#define	MIGR_ON			1	// Migration ON, it is used the Classic Migration Heuristic
#define	MIGR_E1			1	// Classic Migration Heuristic
#define	MIGR_E2			2	// Migration Heuristic 2, based on the last "E2_EVENT_HISTORY_SIZE" events, checked at every timestep
#define	MIGR_E3			3	// Migration Heuristic 3, the same as MIGR_E2 but checked every E3_EVENT_COUNTDOWN events

#define	LOAD_OFF		0	// Load Balancing ON
#define	LOAD_ON			1	// Load Balancing OFF

#define FAKE_OFF		0	// All the interactions are really sent (standard behavior)
#define FAKE_ON			1	// All the interactions are fake (NOT really sent)

/*---------------------------------------------------------------------------*/
/*	P R O T O T Y P E S						     */	
/*---------------------------------------------------------------------------*/

int	GAIA_Initialize ( int, int, char *, char *, char *, int );
void	GAIA_Finalize ( );
void	GAIA_SetFstID ( int );
void	GAIA_SetHistory ( int );
void	GAIA_SetMF ( float );
void	GAIA_SetMT ( unsigned int );
void	GAIA_SetCountDown ( int );
void	GAIA_SetHistorySlots (int );
void	GAIA_SetEventHistorySize (int );
void	GAIA_SetMigration ( int );
void	GAIA_SetFakeInteractions (int );
void	GAIA_SetLoadBalancing ( int );
int	GAIA_Register ( int );
int	GAIA_RegisterState ( int, void *, int );
double	GAIA_GetStep ( );
double	GAIA_TimeAdvance ();
void	GAIA_Migrate ( int, void *, int );
void	GAIA_Send ( int, int, TM_Time, void *, int );
void	GAIA_Broadcast ( int, TM_Time, void *, int );
char	GAIA_Receive ( int *, int *, TM_Time *, void *, int *); 
void	GAIA_GetStatistics ( int *, int *, int *);

/*---------------------------------------------------------------------------*/

#endif /* __GAIA_H */

