#ifndef _RAND_H_
#define _RAND_H_

/* 
   C implementation of Pierre L'ecuyer's "double 
   congruential random number generator" as it appeared in 
   EFFICIENT AND PORTABLE COMBINED RANDOM NUMBER GENERATORS paper in the
   Communications of the ACM, June 1988, pp. 742-774.  

   Based on the ADA implementation of William M. Graham. 
   (Coded by William N. Graham, Applied Research Labs, 2-11-91.)

   Requirements: 32 bit (or larger) machine. 
   The period for this generator is reported to be ~ 2.3 * 10**18. 
*/

#define M1 		2147483563
#define A1 		40014
#define Q1 		53668
#define R1 		12211
#define M2 		2147483399
#define A2 		40692
#define Q2 		52774
#define R2 		3791
#define NORMALIZE 	4.656613E-10
#define TWOPI 		6.28318530718

typedef struct { long S1;  long S2;} TSeed;




int	RND_Init( TSeed*, char *, int );
void	RND_GetSeeds( TSeed*, long*, long* );
void	RND_SetSeeds( TSeed *, long, long );
double	RND_Unif( TSeed*);
double	RND_Interval (TSeed *, double, double );
long 	RND_Integer( TSeed*, long, long );
long 	RND_Binomial( TSeed*, long, double );
double 	RND_Exponential( TSeed*, double );
double 	RND_Gamma( TSeed*, double, double );
long  	RND_Geometric( TSeed*, double );
double 	RND_Normal01( TSeed* );
double 	RND_NormalSD( TSeed*, double, double );
long  	RND_Poisson( TSeed*, double );


#endif /* _RAND_H_ */
