#include <jni.h>
#include "ARTIS.h"
#include "ARTISjava.h"




/***** GAIA *****/

JNIEXPORT jdouble JNICALL Java_ARTISjava_GAIA_1GetStep (JNIEnv *env, jobject obj) {
 	
	return(GAIA_GetStep());  
}


JNIEXPORT jdouble JNICALL Java_ARTISjava_GAIA_1TimeAdvance (JNIEnv *env, jobject obj) {

	return(GAIA_TimeAdvance());
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1GetStatistics (JNIEnv *env, jobject obj) {

	jfieldID	lp_fid, fid1, fid2, fid3;
	jint		lpid, loc, rem, migr;

	
	jclass cls = (*env)->GetObjectClass(env, obj);
	
	lp_fid = (*env)->GetStaticFieldID(env, cls, "LP_ID", "I");
	lpid = (*env)->GetStaticIntField(env, cls, lp_fid);

	fid1 = (*env)->GetStaticFieldID(env, cls, "local", "I");
	fid2 = (*env)->GetStaticFieldID(env, cls, "remote", "I");
	fid3 = (*env)->GetStaticFieldID(env, cls, "migrations", "I");
	
	GAIA_GetStatistics ( &loc, &rem, &migr );

	(*env)->SetStaticIntField(env, cls, fid1, loc);
	(*env)->SetStaticIntField(env, cls, fid2, rem);
	(*env)->SetStaticIntField(env, cls, fid3, migr);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1Finalize (JNIEnv *env, jobject obj) {
	
	GAIA_Finalize();
}


JNIEXPORT jint JNICALL Java_ARTISjava_GAIA_1Initialize (JNIEnv *env, jobject obj, jint max_objs_count, jint nlp, jstring RandFile, jstring cname, jstring sima_host_name, jint sima_port) {

	jfieldID	lp_fid;
	jint		lpid;
	const char /*jbyte*/ *str1, *str2, *str3;


	jclass cls = (*env)->GetObjectClass(env, obj);
	lp_fid = (*env)->GetStaticFieldID(env, cls, "LP_ID", "I");
	
	str1 = (*env)->GetStringUTFChars(env, RandFile, NULL);
	str2 = (*env)->GetStringUTFChars(env, cname, NULL);
	str3 = (*env)->GetStringUTFChars(env, sima_host_name, NULL);

	lpid = GAIA_Initialize(max_objs_count, nlp, (char *)str1, (char *)str2, (char *)str3, sima_port);

	(*env)->SetStaticIntField(env, cls, lp_fid, lpid);

	(*env)->ReleaseStringUTFChars(env, RandFile, str1);
	(*env)->ReleaseStringUTFChars(env, cname, str2);
	(*env)->ReleaseStringUTFChars(env, sima_host_name, str3);


	return lpid;
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1Send (JNIEnv *env, jobject obj, jint from, jint to, jdouble Ts, jstring Msg, jint Size) {

	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, Msg, NULL);
	
	GAIA_Send(from, to, Ts, (void *)str1, Size);

	(*env)->ReleaseStringUTFChars(env, Msg, str1);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1ByteSend (JNIEnv *env, jobject obj, jint from, jint to, jdouble Ts, jbyteArray Msg, jint Size) {

	jbyte *body = (*env)->GetByteArrayElements(env, Msg, 0);
	
	GAIA_Send(from, to, Ts, (void *)body, Size);	
	
	(*env)-> ReleaseByteArrayElements(env, Msg, (jbyte *)body, 0);
    
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1Broadcast (JNIEnv *env, jobject obj, jint from, jdouble Ts, jstring Msg, jint Size) {

	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, Msg, NULL);
	
	GAIA_Broadcast(from, Ts, (void *)str1, Size);

	(*env)->ReleaseStringUTFChars(env, Msg, str1);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1ByteBroadcast (JNIEnv *env, jobject obj, jint from, jdouble Ts, jbyteArray Msg, jint Size) {
	
	jbyte *body = (*env)->GetByteArrayElements(env, Msg, 0);

	GAIA_Broadcast(from, Ts, (void *)body, Size);
	
	(*env)-> ReleaseByteArrayElements(env, Msg, (jbyte *)body, 0);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1SetLoadBalancing (JNIEnv *env, jobject obj, jint state) {
	
	GAIA_SetLoadBalancing(state);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1SetMigration (JNIEnv *env, jobject obj, jint state) {
	
	GAIA_SetMigration(state);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1SetMF (JNIEnv *env, jobject obj, jfloat migration_factor) {
	
	GAIA_SetMF(migration_factor);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1SetMT (JNIEnv *env, jobject obj, jint migration_threshold) {
	
	GAIA_SetMT(migration_threshold);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1SetCountDown (JNIEnv *env, jobject obj, jint countdown) {
	
	GAIA_SetCountDown(countdown);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1SetHistorySlots (JNIEnv *env, jobject obj, jint slots) {
	
	GAIA_SetHistorySlots(slots);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1SetEventHistorySize (JNIEnv *env, jobject obj, jint size) {
	
	GAIA_SetEventHistorySize(size);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1SetFstID (JNIEnv *env, jobject obj, jint first_id) {
	
	GAIA_SetFstID(first_id);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1SetHistory (JNIEnv *env, jobject obj, jint new_size) {
	
	GAIA_SetHistory(new_size);
}


JNIEXPORT jint JNICALL Java_ARTISjava_GAIA_1Register (JNIEnv *env, jobject obj, jint migrable) {
	
	return(GAIA_Register(migrable));
}


JNIEXPORT jstring JNICALL Java_ARTISjava_GAIA_1Receive (JNIEnv *env, jobject obj, jint maxlen) {

	jint	from, to, type;
	jdouble Ts;
	char	datas[maxlen];
	jint	retvalue;
	
	jfieldID	fid1, fid2, fid3, fid4, fid5;
	jclass cls = (*env)->GetObjectClass(env, obj);
	
	fid1 = (*env)->GetStaticFieldID(env, cls, "from", "I");
	fid2 = (*env)->GetStaticFieldID(env, cls, "to", "I");
	fid3 = (*env)->GetStaticFieldID(env, cls, "Ts", "D");
	fid4 = (*env)->GetStaticFieldID(env, cls, "type", "I");
	fid5 = (*env)->GetStaticFieldID(env, cls, "ready", "I");

	retvalue = maxlen;
	type = (int)GAIA_Receive(&from, &to, &Ts, datas, &retvalue);

	(*env)->SetStaticIntField(env, cls, fid1, from);
	(*env)->SetStaticIntField(env, cls, fid2, to);
	(*env)->SetStaticDoubleField(env, cls, fid3, Ts);
	(*env)->SetStaticIntField(env, cls, fid4, type);
	(*env)->SetStaticIntField(env, cls, fid5, retvalue);
		
	datas[retvalue] = '\0';
	
	
	return (*env)->NewStringUTF(env, datas);
}


JNIEXPORT jbyteArray JNICALL Java_ARTISjava_GAIA_1ByteReceive (JNIEnv *env, jobject obj, jint maxlen) {

	jint	from, to, type;
	jdouble Ts;
	char	datas[maxlen];
	jint	retvalue;
	jbyteArray jBytesConverted;
	
	jfieldID	fid1, fid2, fid3, fid4, fid5;
	jclass cls = (*env)->GetObjectClass(env, obj);
	
	fid1 = (*env)->GetStaticFieldID(env, cls, "from", "I");
	fid2 = (*env)->GetStaticFieldID(env, cls, "to", "I");
	fid3 = (*env)->GetStaticFieldID(env, cls, "Ts", "D");
	fid4 = (*env)->GetStaticFieldID(env, cls, "type", "I");
	fid5 = (*env)->GetStaticFieldID(env, cls, "ready", "I");

	retvalue = maxlen;
	type = (int)GAIA_Receive(&from, &to, &Ts, datas, &retvalue);

	(*env)->SetStaticIntField(env, cls, fid1, from);
	(*env)->SetStaticIntField(env, cls, fid2, to);
	(*env)->SetStaticDoubleField(env, cls, fid3, Ts);
	(*env)->SetStaticIntField(env, cls, fid4, type);
	(*env)->SetStaticIntField(env, cls, fid5, retvalue);
		
	datas[retvalue] = '\0';
	
	jBytesConverted = (*env)->NewByteArray(env, retvalue);
	(*env)->SetByteArrayRegion(env, jBytesConverted, 0, retvalue, (jbyte *)datas);
	
	return jBytesConverted;
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1Migrate (JNIEnv *env, jobject obj, jint obj_id, jstring state, jint state_size) {

	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, state, NULL);
	
	GAIA_Migrate(obj_id, (void *)str1, state_size);

	(*env)->ReleaseStringUTFChars(env, state, str1);
}


JNIEXPORT void JNICALL Java_ARTISjava_GAIA_1ByteMigrate (JNIEnv *env, jobject obj, jint obj_id, jbyteArray state, jint state_size) {

	jbyte *body = (*env)->GetByteArrayElements(env, state, 0);
	
	GAIA_Migrate(obj_id, (void *)body, state_size);
	
	(*env)-> ReleaseByteArrayElements(env, state, (jbyte *)body, 0);
}


/***** TimeStepped *****/

JNIEXPORT jdouble JNICALL Java_ARTISjava_TS_1GetStep (JNIEnv *env, jobject obj) {
 	
	return(TS_GetStep());  
}


JNIEXPORT jdouble JNICALL Java_ARTISjava_TS_1TimeAdvance (JNIEnv *env, jobject obj) {

	return(TS_TimeAdvance());
}


JNIEXPORT void JNICALL Java_ARTISjava_TS_1Finalize (JNIEnv *env, jobject obj) {
	
	TS_Finalize();
}


JNIEXPORT jint JNICALL Java_ARTISjava_TS_1Init (JNIEnv *env, jobject obj, jstring cname, jstring sima_host_name, jint sima_port) {

	jint	lpid;
	const char /*jbyte*/ *str1, *str2;
	
	str1 = (*env)->GetStringUTFChars(env, cname, NULL);
	str2 = (*env)->GetStringUTFChars(env, sima_host_name, NULL);
	
	lpid = TS_Init((char *)str1, (char *)str2, sima_port);

	(*env)->ReleaseStringUTFChars(env, cname, str1);
	(*env)->ReleaseStringUTFChars(env, sima_host_name, str2);

	return(lpid);
}


JNIEXPORT jint JNICALL Java_ARTISjava_TS_1Send (JNIEnv *env, jobject obj, jint lp, jdouble Ts, jstring Msg, jint Size) {
	
	jint	ret;
	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, Msg, NULL);
	
	ret = TS_Send(lp, Ts, (void *)str1, Size);

	(*env)->ReleaseStringUTFChars(env, Msg, str1);

	return(ret);
}


JNIEXPORT jint JNICALL Java_ARTISjava_TS_1Broadcast (JNIEnv *env, jobject obj, jdouble Ts, jstring Msg, jint Size) {
	
	jint	ret;
	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, Msg, NULL);
	
	ret = TS_Broadcast(Ts, (void *)str1, Size);

	(*env)->ReleaseStringUTFChars(env, Msg, str1);

	return(ret);
}


JNIEXPORT jint JNICALL Java_ARTISjava_TS_1SendToOthers (JNIEnv *env, jobject obj, jdouble Ts, jstring Msg, jint Size) {

	jint	ret;
	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, Msg, NULL);
	
	ret = TS_SendToOthers(Ts, (void *)str1, Size);

	(*env)->ReleaseStringUTFChars(env, Msg, str1);

	return(ret);
}


JNIEXPORT jint JNICALL Java_ARTISjava_TS_1Schedule (JNIEnv *env, jobject obj, jdouble Ts, jstring Msg, jint Size) {

	jint	ret;
	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, Msg, NULL);
	
	ret = TS_Schedule(Ts, (void *)str1, Size);

	(*env)->ReleaseStringUTFChars(env, Msg, str1);

	return(ret);
}


JNIEXPORT jstring JNICALL Java_ARTISjava_TS_1Receive (JNIEnv *env, jobject obj, jint maxlen) {

	jint	from;
	jdouble Ts;
	char	datas[maxlen];
	jlong	retvalue;
	
	jfieldID	fid1, fid2, fid3;
	jclass cls = (*env)->GetObjectClass(env, obj);
	
	fid1 = (*env)->GetStaticFieldID(env, cls, "from", "I");
	fid2 = (*env)->GetStaticFieldID(env, cls, "Ts", "D");
	fid3 = (*env)->GetStaticFieldID(env, cls, "ready", "I");
	

	retvalue = TS_Receive(&from, &Ts, datas, maxlen);
	
	(*env)->SetStaticIntField(env, cls, fid3, retvalue);
	
	if (retvalue >0) {
		(*env)->SetStaticIntField(env, cls, fid1, from);
		(*env)->SetStaticDoubleField(env, cls, fid2, Ts);
		
		datas[retvalue] = '\0';
		
		return (*env)->NewStringUTF(env, datas);
	}
	else 
		return (*env)->NewStringUTF(env, "");   
}


/***** CMB *****/

JNIEXPORT jint JNICALL Java_ARTISjava_CMB_1Init (JNIEnv *env, jobject obj, jstring cname, jstring sima_host_name, jint sima_port) {

	jint	lpid;
	const char /*jbyte*/ *str1, *str2;
	
	str1 = (*env)->GetStringUTFChars(env, cname, NULL);
	str2 = (*env)->GetStringUTFChars(env, sima_host_name, NULL);
	
	lpid = CMB_Init((char *)str1, (char *)str2, sima_port);

	(*env)->ReleaseStringUTFChars(env, cname, str1);
	(*env)->ReleaseStringUTFChars(env, sima_host_name, str2);

	return(lpid);
}


JNIEXPORT jint JNICALL Java_ARTISjava_CMB_1Send (JNIEnv *env, jobject obj, jint lp, jdouble Ts, jstring Msg, jint Size) {
	
	jint	ret;
	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, Msg, NULL);
	
	ret = CMB_Send(lp, Ts, (void *)str1, Size);

	(*env)->ReleaseStringUTFChars(env, Msg, str1);

	return(ret);
}


JNIEXPORT jint JNICALL Java_ARTISjava_CMB_1Broadcast (JNIEnv *env, jobject obj, jdouble Ts, jstring Msg, jint Size) {

	jint	ret;
	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, Msg, NULL);
	
	ret = CMB_Broadcast(Ts, (void *)str1, Size);

	(*env)->ReleaseStringUTFChars(env, Msg, str1);

	return(ret);
}


JNIEXPORT jint JNICALL Java_ARTISjava_CMB_1SendToOthers (JNIEnv *env, jobject obj, jdouble Ts, jstring Msg, jint Size) {

	jint	ret;
	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, Msg, NULL);
	
	ret = CMB_SendToOthers(Ts, (void*)str1, Size);

	(*env)->ReleaseStringUTFChars(env, Msg, str1);

	return(ret);
}


JNIEXPORT jint JNICALL Java_ARTISjava_CMB_1Schedule (JNIEnv *env, jobject obj, jdouble Ts, jstring Msg, jint Size) {

	jint	ret;
	const char /*jbyte*/ *str1;
	
	str1 = (*env)->GetStringUTFChars(env, Msg, NULL);
	
	ret = CMB_Schedule(Ts, (void *)str1, Size);

	(*env)->ReleaseStringUTFChars(env, Msg, str1);

	return(ret); 
}


JNIEXPORT void JNICALL Java_ARTISjava_CMB_1Finalize (JNIEnv *env, jobject obj) {

	CMB_Finalize();
}


JNIEXPORT jint JNICALL Java_ARTISjava_CMB_1GetLookAhead (JNIEnv *env, jobject obj, jint lp) {

	return(CMB_GetLookAhead(lp));
}



JNIEXPORT jdouble JNICALL Java_ARTISjava_CMB_1SetLookAhead (JNIEnv *env, jobject obj, jint lp, jdouble NewLA) {

	return(CMB_SetLookAhead(lp, NewLA));
}


JNIEXPORT jstring JNICALL Java_ARTISjava_CMB_1Receive (JNIEnv *env, jobject obj, jint maxlen) {
	
	jint	from;
	jdouble Ts;
	char	datas[maxlen];
	
	jlong 	 retvalue;
	jfieldID fid1, fid2, fid3;

	jclass cls = (*env)->GetObjectClass(env, obj);
	

	fid1 = (*env)->GetStaticFieldID(env, cls, "from", "I");
	fid2 = (*env)->GetStaticFieldID(env, cls, "Ts", "D");
	fid3 = (*env)->GetStaticFieldID(env, cls, "ready", "I");
	
	retvalue = CMB_Receive(&from, &Ts, datas, maxlen);
	
	(*env)->SetStaticIntField(env, cls, fid3, retvalue);
	
	if (retvalue >0) {
	
		(*env)->SetStaticIntField(env, cls, fid1, from);
		(*env)->SetStaticDoubleField(env, cls, fid2, Ts);
		
		datas[retvalue] = '\0';
		
		return (*env)->NewStringUTF(env, datas);
	}
	else 
		return (*env)->NewStringUTF(env, "");   
}
