/*      ##############################################################################################
        Advanced RTI System, ARTÌS                      http://pads.cs.unibo.it

        Description:
                -       In this file is defined the state of simulated entities

        Authors:
                First version by Gabriele D'Angelo <gda@cs.unibo.it>

        ############################################################################################### */

#ifndef __ENTITY_DEFINITION_H
#define __ENTITY_DEFINITION_H

/*---- E N T I T Y    D E F I N I T I O N ---------------------------------*/

// The "value" that is associated to a "key" in practice can be implemented
//	as one or more values
typedef struct v_e {
	unsigned int	value;
	/* ... */
} value_element;

// Records composing the local state (dynamic part) of each SE
// 	NOTE: no duplicated keys are allowed
struct state_element {
	unsigned int	key;			// key of the entry
	value_element	elements;		// "value" associated to the key
};

// Static part of the SE state
typedef struct static_data_t {
        char                    changed;                        // ON if there has been a state change in the last timestep
} static_data_t;

// SE state definition
typedef struct hash_data_t {
        int                     key;                            // SE identifier
        int                     lp;                             // Logical Process ID
        static_data_t           s_state;                        // Static part of the SE local state
        GHashTable*             state;                          // Local state as an hash table (glib) (dynamic part)
								//	NOTE: each element in the hash_table is has a 
								//		"state_element" structure
} hash_data_t;

#endif /* __ENTITY_DEFINITION_H */

