/*	##############################################################################################
	Advanced RTI System, ARTÌS			http://pads.cs.unibo.it

	Description:
		-	In this file are defined the message types and its structures

	Authors:
		First version by Gabriele D'Angelo <gda@cs.unibo.it>

	############################################################################################### */

#ifndef __MESSAGE_DEFINITION_H
#define __MESSAGE_DEFINITION_H

#include "entity_definition.h"

/*---- M E S S A G E S    D E F I N I T I O N ---------------------------------*/

// Model messages definition
typedef struct _ping_msg    	PingMsg;
typedef struct _link_msg    	LinkMsg;
typedef struct _migr_msg    	MigrMsg;
typedef union   msg         	Msg;

// General note:
//	each type of message is composed of a static and a dynamic part
//	-	the static part contains a pre-defined set of variables, and the size
//		of the dynamic part (as number of records)
//	-	a dynamic part that is composed of a sequence of records


// **********************************************
// PING MESSAGES
// **********************************************
// Record definition for dynamic part of ping messages
//	each record is composed of a "key" and a "value"
struct _ping_record {
	unsigned int	key;
	unsigned int	value;
};
//
// Static part of ping messages
struct _ping_static_part {
	char		type;				// Message type
	unsigned int	dyn_records;			// Number of records in the dynamic part of the message
};
//
// Dynamic part of ping messages
//	it is composed of a set of records
struct _ping_dynamic_part {
	struct _ping_record	records[MAX_PING_DYNAMIC_RECORDS];
};
//
// Ping message
struct _ping_msg {
	struct	_ping_static_part		ping_static;	// Static part
	struct	_ping_dynamic_part		ping_dynamic;	// Dynamic part
};


// **********************************************
// MIGRATION MESSAGES
// **********************************************
//
// Static part of migration messages
struct _migration_static_part {
	char		type;							// Message type
        static_data_t   s_state;                                                // Static part of the SE state: it is the same
                                                                                //       of the static part of the simulated entities state
 	unsigned int	dyn_records;						// Number of records in the dynamic part of the message
};
//
// Dynamic part of ping messages
struct _migration_dynamic_part {
	struct state_element	records[MAX_MIGRATION_DYNAMIC_RECORDS];
};
//
// Migration message
struct _migr_msg {
	struct	_migration_static_part		migration_static;
	struct	_migration_dynamic_part		migration_dynamic;
};

/////////////////////////////////////////////////
// Union structure for messages
union msg {
	char		type;
    	PingMsg		ping;
	MigrMsg		migr;
};
/*---------------------------------------------------------------------------*/

#endif /* __MESSAGE_DEFINITION_H */

